AC_DEFUN([AX_CUDA], 
[
ax_cuda_ok=no
AC_ARG_WITH(cuda,
    [AC_HELP_STRING([--with-cuda=<path>], [use cudatoolkit path])])

AS_IF([test -z "$with_cuda" && test "${with_cuda+set}" = set ],
      [AC_MSG_WARN([--with-cuda=<EMPTY>])],[])


AS_CASE(["$with_cuda"],
        [yes],[ax_cuda_ok=yes],
        [no|""],[ax_cuda_ok=no],
        [-* | *.a | *.so | *.so.* | *.o],[AC_MSG_ERROR([Apply PATH to cudatoolkit or set yes])],
        [CUDA_PATH="$with_cuda"; ax_cuda_ok=yes])

AS_IF([test "x$ax_cuda_ok" = xyes],
      [AS_IF([test -z "$CUDA_PATH"],
             [AC_PATH_PROG([NVCC],[nvcc],[notfound])
              AS_IF([test "$NVCC" = notfound],
                    [AC_MSG_ERROR([nvcc not found])],[])
              CUDA_PATH=${NVCC%/bin/nvcc}],[])
       AC_MSG_CHECKING([linking CUDA])
       CUDA_PATH=${CUDA_PATH%/bin//nvcc};
       OLDCFLAGS="$CFLAGS";
       CFLAGS+=" -I$CUDA_PATH/include";
       OLDLIBS="$LIBS"
       LIBS+=" -L$CUDA_PATH/lib -L$CUDA_PATH/lib64 -lcuda -lcudart -lcublas"
       AC_LANG_PUSH([C])
       m4_define([AX_CHECK_CUDA_PROGRAM],
                 [AC_LANG_PROGRAM([[
#include <cuda.h>
#include <cuda_runtime_api.h>
#include <cublas.h>]],         
                 [[cuInit(0);cudaSetDevice(0);cublasInit();]])])
       AC_LINK_IFELSE([AX_CHECK_CUDA_PROGRAM],
                      [AC_MSG_RESULT([ok])],
                      [AC_MSG_RESULT([failed]);AC_MSG_FAILURE([--with-cuda was given, but linking failed])])
       AC_LANG_POP([C])
       LIBS="$OLDLIBS"],[])
])

AC_DEFUN([AX_MAGMA],
[
ax_magma_ok=no
AC_ARG_WITH(magma,
	[AC_HELP_STRING([--with-magma=<lib>], [use magma library <lib>])])
	
AS_IF([test -z "$with_magma" && test "${with_magma+set}" = set ],
      [AC_MSG_WARN([--with-magma=<EMPTY>])],[])
	
AS_CASE(["$with_magma"],
	    [yes],[ax_magma_ok=yes],
	    [no|""],[ax_magma_ok=disable],
	    [-*|*/*|*.a|*.so|*.so.*|*.o],[MAGMA_LIBS="$with_magma";ax_magma_ok=yes],
	    [MAGMA_LIBS="-l$with_magma"; ax_magma_ok=yes])
]) 

AC_DEFUN([AX_CUDPP],
[
ax_cudpp_ok=no
AC_ARG_WITH(cudpp,
	[AC_HELP_STRING([--with-cudpp=<path>], [use cudpp path, containing libcudpp.a and cudpp.h])])
	
AS_IF([test -z "$with_cudpp" && test "${with_cudpp+set}" = set ],
      [AC_MSG_WARN([--with-cudpp=<EMPTY>])],[])

AS_CASE(["$with_cudpp"],
	    [yes],[ax_cudpp_ok=yes],
	    [no|""],[ax_cudpp_ok=disable],
	    [-*|*.a|*.so|*.so.*|*.o],[AC_MSG_ERROR([Apply PATH to cudpp ])],
	    [CUDPP_LIBS="$with_cudpp";ax_cudpp_ok=yes])
]) 


