/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2011: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef BUCKETS_H
#define BUCKETS_H

#define BUCKET_SPATIAL_BITS 10
#define BUCKET_SPATIAL_X  0
#define BUCKET_SPATIAL_Y 10
#define BUCKET_SPATIAL_Z 20
#define BUCKET_SPATIAL_M 1000
#define BUCKET_SPATIAL_MASK 0x3FF

#define BUCKET_DIM 3

/*! Move vector and phatom mask */
#define MVP_MASK 15
/*! Number of bits occupied by the mask */
#define MVP_BITS 4
/*! X coordinate mask */
#define MVP_X 1
#define MVP_Xb 0
/*! Y coordinate mask */
#define MVP_Y 2
#define MVP_Yb 1
/*! Z coordinate mask */
#define MVP_Z 4
#define MVP_Zb 2
/*! Phantom coordinate mask */
#define MVP_PH 8
#define MVP_PHb 3

#define MAX_PHANTOMS 20

#include "defs.h"

#ifdef __cplusplus
extern "C" {
#endif

/*! Algorith to generate interaction list in spatial subdivision. */
extern BUCKET_NONE_SPATIAL nb_list;
MAKE_STR_DEC(BUCKET_NONE_SPATIAL,nb_list)

typedef struct
{
    /*! Cell size in each dimension */
    DOUBLE blen[3];
    DOUBLE l[3];
    INT size_keys;
    key_pid* keys;
    INT* ids;
    INT size_ids;

    INT max_beads;
    /*! Number of disjoint groups*/
    INT max_group;
    /*! Temporary table for PDEs, for sorting */
    key_pid* tmp;
    /*! Local version of coords that should be get from global 
     * each time, 2D array, for each thread copy of local coords */
    DOUBLE** coords;
    /*! Local version of forces that should be cleaned
     * each time, 2D array, for each thread copy of local forces,
     * after computation forces should be added to global forces array */
    DOUBLE** F;
    INT* capacity;
} bucket_s;

typedef enum
{
    electro_type, electro_desolv_type, lj_type, overlap_type, hdb_type, last_type
} bucket_keys_type;

DOUBLE* get_tmp_coords( bucket_s* bs, INT tid, INT LD_COORD, INT LD_F );
DOUBLE* get_tmp_forces( bucket_s* bs, INT tid, INT LD_COORD, INT LD_F );

INT pre_group( INT key );
void init_bucket( bucket_s** pbr, int size, INT LD_COORD, INT LD_F );
void free_bucket( bucket_s* );

INT init_keys( bucket_s* br, INT size, const PTR_DOUBLE _coords, DOUBLE blen, INT LD, INT (*ignore)(const DOUBLE*) );
INT init_keys_end( bucket_s* br );
void gen_keys( bucket_s* br, const PTR_DOUBLE _coords, INT id, INT (*ignore)(const DOUBLE*) );
void gen_keys_p( bucket_s* br, const PTR_DOUBLE _coords, INT id, INT (*ignore)(const DOUBLE*) );
INT can_interact( INT pidn, INT pidm );

INT make_keyb(INT x,INT y,INT z);
INT get_h_l( INT size, const PTR_DOUBLE coords, DOUBLE h[3], DOUBLE l[3], INT LD, PTR_DOUBLE blen );

INT in_phan_reg_down(DOUBLE x, INT dim, DOUBLE egde_len);
INT in_phan_reg_up(DOUBLE x, INT dim, DOUBLE egde_len);
INT correct_indx(int indx, int last);
int cmp_keys(const void* a, const void* b);
UINT gen_hash_key( INT key );

VOID correct_xyz( DOUBLE* v );

#ifdef USE_MPI
extern INT g_buck_len[DIMS0];
extern INT g_max_x;
extern INT g_max_xy;
INT can_add( INT key );
#endif

#ifdef __cplusplus
}
#endif

#endif/* BUCKET_H */
