/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@cent.uw.edu.pl>        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#include "cuda_common.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "err.h"

MAKE_STR_IN(PTR_INT,cuda_devices,NULL,"Cuda devices to be used")
MAKE_STR_IN(INT,cuda_block,256,"The dimension of the thread block")

#if USE_CUDA
#include <cuda_runtime_api.h>

INT sm_red;
INT OLDARCH;

void cudaCheckError( const char* file, int line )
{
    cudaError_t error = cudaGetLastError();
    if( error != 0 )
    {
        pre_warning(file,line);
        printlog_var("CUDA error: %s", cudaGetErrorString(error) );
        UNERRDW
    }
}

void cublasCheckError( const char* file, int line )
{
    INT error = cublasGetError();
    if( error != 0 )
    {
        warning( "CUBLAS error", file, line );
        UNERR("End execution due to cuda error")
    }
}

void init_cuda_common()
{
    INT id = 0;
    struct cudaDeviceProp deviceProp;
    if( cuda_devices == NULL )
    {
        g_devices = 1;
        cudaSetDevice( 0 ); CCERR
    }
    else
    {
        if( variable_cuda_devices.size == 1 )
        {
            id = cuda_devices[0];
            cudaSetDevice( cuda_devices[0] ); CCERR
        }
        else
        {
            UNERR("Unimplemented, only single GPU supported");
        }
    }
    cudaGetDeviceProperties( &deviceProp, id ); CCERR
    sm_red = deviceProp.multiProcessorCount;
    if ( sm_red < 6 ) sm_red = 6; /* TODO make constant value */
    if ( deviceProp.maxThreadsPerBlock < cuda_block )
    {
        WARNV(("Wrong size of cuda_block, max %d, current %d",deviceProp.maxThreadsPerBlock,cuda_block))
        UNERRDW
    }

    OLDARCH = 0;
    if ( deviceProp.major <= 1 && deviceProp.minor < 3 ) OLDARCH = 1;
}

void free_cuda_common()
{
    cudaThreadExit(); CCERR
}

void show_array( float* array, int count, const char* name )
{
    if ( name ) printf( "%s ", name );
    if ( array )
    {
        int csf = count * sizeof(float);
        int i;
        float* td = (float*) malloc( csf ); CHMEM(td);
        cudaMemcpy( td, array, csf, cudaMemcpyDeviceToHost );
        for ( i = 0; i < count; ++i )
        {
            printf( "%g ", td[i] );
        }
        free(td);
    }
    else
    {
        printf("NULL");
    }
    printf("\n");
}

#endif
