/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2011: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@cent.uw.edu.pl>        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef CUDA_COMMON_H
#define	CUDA_COMMON_H

#include "defs.h"

#ifdef	__cplusplus
extern "C" {
#endif

/*! Array of devices in OS to use. */
extern PTR_INT cuda_devices;
MAKE_STR_DEC(PTR_INT,cuda_devices)

/*! Size of block of threads in kernel. */
extern INT cuda_block;
MAKE_STR_DEC(INT,cuda_block)

#if USE_CUDA || USE_MAGMA

/*! Number of streaming processors in device. */
extern INT sm_red;
extern INT OLDARCH;

/*! Print on stdout float array from CUDA device.
    \param array array to print
    \param number of elements to print
    \param name optional name of array */
void show_array( float* array, int count, const char*name );
#endif

/*! Macro for checking errors from device. */
#define CCERR cudaCheckError( __FILE__, __LINE__ );

/*! Help function to check if cuda error ocurred */
void cudaCheckError( const char* file, int line );

/*! Help function to check if cudablas error ocurred */
void cublasCheckError( const char* file, int line );

/*! Initializes CUDA device and allocs memory. */
void init_cuda_common();

/*! Free memory and other resources used in CUDA. */
void free_cuda_common();

#ifdef	__cplusplus
}
#endif

#endif	/* CUDA_COMMON_H */
