/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef ERR_H
#define ERR_H

#include "defs.h"

/*! Default name of error log file. */
#define ERR_FILENAME "err.log"

#ifdef	__cplusplus
extern "C" {
#endif

/*! Name of plain text log file (output). */
extern STR out_filename;
MAKE_STR_DEC(STR,out_filename)

#define UNERR(mssg) unexpected_error( mssg, __FILE__, __LINE__ );
#define WARN(mssg) warning( mssg, __FILE__, __LINE__ );
#define LOG(mssg) logmssg( mssg, __FILE__, __LINE__ );
#define UNERRDW UNERR("Exiting with failure status due to previous warning")
        
/*! Unsafe */
#define WARNV(_args_) {pre_warning(__FILE__,__LINE__); printlog_var _args_ ;}
#define LOGV(_args_) {pre_log(__FILE__,__LINE__);printlog_var _args_ ;}

/*! Print error to log file and end program with error message.
    \param mssg Message to print
    \param file name of source file
    \param line number of line in source file*/
void unexpected_error( CSTR mssg, CSTR file, INT line );

/*! Print warning to log file.
    \param mssg Message to print
    \param file name of source file
    \param line number of line in source file */
void warning( CSTR mssg, CSTR file, INT line );

/*! Print log-message to log file.
    \param mssg Message to print
    \param file name of source file
    \param line number of line in source file*/
void logmssg( CSTR mssg, CSTR file, INT line );

/*! Print warning header to log file.
    \param mssg Message to print
    \param file name of source file
    \param line number of line in source file */
void pre_warning( CSTR file, INT line );

/*! Print log-message header to log file.
    \param mssg Message to print
    \param file name of source file
    \param line number of line in source file*/
void pre_log( CSTR file, INT line );

/*! Print message body to log file.
    \param mssg Message to print
    \param file name of source file
    \param line number of line in source file*/
void printlog_var( CSTR format, ... );

/*! Check if memory was allocated, if not end program.
    \param ptr pointer to check
    \param file name of source file
    \param line number of line in source file*/
void checkMem( void* ptr, CSTR file, INT line );

#define CHMEM(ptr) checkMem( ptr, __FILE__, __LINE__ );

/*! Free resources, flush and close log file. */
void free_logfile();

/*! Help function to add suffix on end of copy of filename.
    \param filename to copy
    \param suffix string to add to filename, bu before extension*/
STR add_suffix( CSTR filename, CSTR suffix );

#ifdef	__cplusplus
}
#endif

#endif	/* ERR_H */
