/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2011 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@cent.uw.edu.pl>        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#include "htablekp.h"

#include <stdlib.h>

#include "err.h"

hash_table_kp* create_hash_table_kp( INT size, INT icap )
{
    INT i;
    hash_table_kp* res;
    if ( icap <= 0 ) return NULL;
    res = (hash_table_kp*) malloc( sizeof( hash_table_kp ) ); CHMEM(res)
    res->entrs = (hash_entry_kp*) malloc( size * sizeof( hash_entry_kp ) );
    res->size = size;
    for ( i = 0; i < size; ++i )
    {
        res->entrs[i].cap = icap;
        res->entrs[i].end = 0;
        res->entrs[i].vals = (key_pid*) malloc( icap * sizeof(key_pid) );
        CHMEM(res->entrs[i].vals);
    }
    return res;
}

void free_hash_table_kp( hash_table_kp* ht)
{
    INT i;
    for ( i = 0; i < ht->size; ++i )
    {
        free(ht->entrs[i].vals);
    }
    free(ht->entrs);
    free(ht);
}

void add_to_hash_table_kp( hash_table_kp* ht, key_pid kp )
{
    INT pos = kp.key % ht->size;
    key_pid* vals = ht->entrs[ pos ].vals;
    if ( ht->entrs[pos].end >= ht->entrs[pos].cap )
    {
        INT ncap = 2 * ht->entrs[ pos ].cap + 1;
        ht->entrs[ pos ].cap = ncap;
        vals = (key_pid*) realloc( vals, ncap * sizeof(key_pid) );
        CHMEM(vals);
    }
    vals[ ht->entrs[pos].end ] = kp;
    ht->entrs[pos].end++;
    ht->entrs[pos].vals = vals;
}

hash_entry_kp* find_elem_ht_kp( hash_table_kp* ht, INT key )
{
    return ht->entrs + (key % ht->size);
}
