/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@cent.uw.edu.pl>        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#include "math_common.h"

#include <math.h>

DOUBLE inv_L[4];

DOUBLE sq3v(DOUBLE* v)
{
    return v[0]*v[0] + v[1]*v[1] + v[2]*v[2];
}

DOUBLE norm3v(DOUBLE* v)
{
    return SQRTD( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );
}

void trans_to_box( DOUBLE* v )
{
    if( bc != BC_BOX ) return;
    trans_to_box_force( v );
}

void trans_to_box_force( DOUBLE* v )
{
    v[0] -= box[0] * FLOORD( v[0] * inv_L[0] + 0.5f );
    v[1] -= box[1] * FLOORD( v[1] * inv_L[1] + 0.5f );
    v[2] -= box[2] * FLOORD( v[2] * inv_L[2] + 0.5f );
}

void trans_dist_vec( DOUBLE* v1, DOUBLE* v2, DOUBLE* v )
{
    v[0] = v2[0] - v1[0];
    v[1] = v2[1] - v1[1];
    v[2] = v2[2] - v1[2];
    if( bc != BC_BOX ) return;
    /* function driven */
    v[0] -= box[0] * FLOORD( v[0] * inv_L[0] + 0.5f );
    v[1] -= box[1] * FLOORD( v[1] * inv_L[1] + 0.5f );
    v[2] -= box[2] * FLOORD( v[2] * inv_L[2] + 0.5f );
}

void get_v2_sq( DOUBLE* v1, DOUBLE* v2, DOUBLE* r_sq )
{
    DOUBLE e1[DIMS0];
    trans_dist_vec( v1, v2, e1 );
    *r_sq = sq3v(e1);
}

void get_v2_sq_force( DOUBLE* v1, DOUBLE* v2, DOUBLE* r_sq )
{
    DOUBLE v[DIMS0];
    v[0] = v2[0] - v1[0];
    v[1] = v2[1] - v1[1];
    v[2] = v2[2] - v1[2];
    trans_to_box_force( v );
    *r_sq = sq3v(v);
}

void get_v2_norms( DOUBLE* v1, DOUBLE* v2, DOUBLE* e1, DOUBLE* r_norm, DOUBLE* r_sq )
{
    DOUBLE inv = 0.0;

    e1[0] = v2[0] - v1[0];
    e1[1] = v2[1] - v1[1];
    e1[2] = v2[2] - v1[2];
    trans_to_box(e1);

    *r_sq = sq3v(e1);
    *r_norm = SQRTD( *r_sq );
    inv = 1 / *r_norm;

    e1[0] *= inv;
    e1[1] *= inv;
    e1[2] *= inv;
}


void get_v2_norms_no( DOUBLE* v1, DOUBLE* v2, DOUBLE* e1, DOUBLE* r_norm, DOUBLE* r_sq )
{
    DOUBLE inv = 0.0;

    e1[0] = v2[0] - v1[0];
    e1[1] = v2[1] - v1[1];
    e1[2] = v2[2] - v1[2];

    *r_sq = sq3v(e1);
    *r_norm = SQRTD( *r_sq );
    inv = 1 / *r_norm;

    e1[0] *= inv;
    e1[1] *= inv;
    e1[2] *= inv;
}

void get_ev2_norms( DOUBLE x1, DOUBLE y1, DOUBLE z1,
                    DOUBLE* e1, DOUBLE* r_norm, DOUBLE* r_sq )
{
    DOUBLE inv = 0.0;
    e1[0] = x1;
    e1[1] = y1;
    e1[2] = z1;
    
    *r_sq = (x1)*(x1) + (y1)*(y1) + (z1)*(z1);
    *r_norm = SQRTD( *r_sq );
    inv = 1 / *r_norm;

    e1[0] *= inv;
    e1[1] *= inv;
    e1[2] *= inv;
}

DOUBLE cos_vs( DOUBLE* e1, DOUBLE* e2 )
{
    DOUBLE cos_v = (e1[0]*e2[0]+e1[1]*e2[1]+e1[2]*e2[2]);
    if( cos_v <= -1.0 ) cos_v = -1.0;
    if( cos_v >=  1.0 ) cos_v =  1.0;
    return cos_v;
}

DOUBLE ABSD(DOUBLE val)
{
    return val < 0.0 ? -val : val;
}

DOUBLE MAXD(DOUBLE v1, DOUBLE v2)
{
    return v1 < v2 ? v2 : v1;
}

DOUBLE MIND(DOUBLE v1, DOUBLE v2)
{
    return v1 > v2 ? v2 : v1;
}

void cross_product(DOUBLE*a,DOUBLE*b,DOUBLE*o)
{
    o[0] = a[1]*b[2]-a[2]*b[1];
    o[1] = a[2]*b[0]-a[0]*b[2];
    o[2] = a[0]*b[1]-a[1]*b[0];
}

DOUBLE SINGD(DOUBLE a)
{
    if( a == 0.0 ) return 0.0;
    if( a < 0.0 ) return -1.0;
    return 1.0;
}

#ifndef HAVE_ISNAN
int isnan(DOUBLE a)
{
    return a!=a;
}
#endif

#ifndef HAVE_ISINF
int isinf(DOUBLE a)
{
    DOUBLE zero_ = 0.0f;
    DOUBLE inf_ = 1/zero_;
    if ( a < 0 ) return (inf_ == -a);
    return (inf_ == a);
}
#endif
