/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef VERLET_H
#define VERLET_H

#define BUCKET_SPATIAL_BITS 10
#define BUCKET_SPATIAL_X  0
#define BUCKET_SPATIAL_Y 10
#define BUCKET_SPATIAL_Z 20
#define BUCKET_SPATIAL_M 1000
#define BUCKET_SPATIAL_MASK 0x3FF

#define BUCKET_DIM 3

#define PAIR_CAP 16

/*! Flag ignore group */
#define VS_IG 1
/*! Flag ignore single */
#define VS_IS 2

#include "defs.h"
#include "htablekp.h"

#ifdef __cplusplus
extern "C" {
#endif

/*! Algorith to generate interaction list in spatial subdivision. */
extern INT verlet_count;
MAKE_STR_DEC(INT,verlet_count)

extern DOUBLE verlet_roff;
MAKE_STR_DEC(DOUBLE,verlet_roff)

extern INT verlet_dash[][3];
extern INT verlet_dash_size;

typedef struct
{
    INT id, jd;
} pairii;

typedef struct
{
    INT cap;
    INT size;
    pairii* prs;
} pairs;

typedef struct
{
    INT* cap;
    INT* size;
    INT** conns;
} graph;

typedef struct
{
    /*! Cell size in each dimension */
    DOUBLE blen[DIMS0];
    DOUBLE l[DIMS0];
    DOUBLE Rcutoff;
    INT mh[DIMS0];
    INT size;
    INT iter;

    /*! Temporary table for PDEs, for sorting */
    key_pid** tmp;
    /*! Local version of coords that should be get from global 
     * each time, 2D array, for each thread copy of local coords */
    DOUBLE** coords;
    hash_table_kp** ht;
    graph* g;
} verlet_s;

VOID add_pair_g( graph* ps, INT id, INT jd );

VOID check_box( verlet_s* vs, graph* g, key_pid* kp1, DOUBLE* crds1, INT s1, INT x, INT y, INT z, DOUBLE* crds2, DOUBLE* _coord, INT LD, INT (*ignore_pair)(key_pid*,key_pid*) );

VOID init_verlet( verlet_s** pbr, int size );
VOID free_verlet( verlet_s* );

INT init_keys_v( verlet_s* br, INT size, const PTR_DOUBLE _coords, DOUBLE blen, INT LD, INT (*ignore)(const DOUBLE*), INT (*ignore_pair)(key_pid*,key_pid*), INT (*can_compute)(INT) );
INT init_keys_end_v( verlet_s* br, const PTR_DOUBLE _coords, INT LD, INT (*ignore_pair)(key_pid*,key_pid*), INT (*can_compute)(INT) );
INT gen_keys_v( verlet_s* br, const PTR_DOUBLE _coords, INT id, INT tid, INT (*ignore)(const DOUBLE*) );

INT make_key_v(INT x,INT y,INT z, verlet_s* vs);
INT get_h_l_v( INT size, const PTR_DOUBLE coords, DOUBLE h[DIMS0], DOUBLE l[DIMS0], INT MI[DIMS0], INT LD, PTR_DOUBLE blen );

INT correct_indx(int indx, int last);
int cmp_keys(const void* a, const void* b);
VOID correct_xyz( DOUBLE* v );

VOID fix_cells( verlet_s* vs, INT* px, INT* py, INT* pz );
#ifdef __cplusplus
}
#endif

#endif/* BUCKET_H */
