/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#include "cuda.h"

#include <stdio.h>
#include <stdlib.h>

#include "diff_alg/diff_tensor.h"
#include "potentials/electro.h"
#include "potentials/LJ.h"

#include "../common/cuda_common.h"
#include "../common/err.h"

#if USE_CUDA || USE_MAGMA
#include <cuda_runtime_api.h>
#include <cublas.h>

DOUBLE* d_coord = NULL;
DOUBLE* d_scoord = NULL;
DOUBLE* d_D = NULL;
INT     size_D = 0;
INT     size_coord = 0;
DOUBLE* d_X = NULL;
DOUBLE* h_X = NULL;
DOUBLE* d_Rs0 = NULL;
DOUBLE* h_Rs0 = NULL;
DOUBLE* d_sRs0 = NULL;
DOUBLE* d_sforces = NULL;
DOUBLE* h_sforces = NULL;
DOUBLE* d_forces = NULL;
DOUBLE* h_forces = NULL;

DOUBLE* d_acoord = NULL;
DOUBLE* d_aRs0 = NULL;
DOUBLE* d_aD = NULL;

INT cblocks;
INT csize;

void init_cuda()
{
    init_cuda_common();
    
    cublasInit(); CCERR
    
    cblocks = ( size + cuda_block - 1 ) / cuda_block;
    csize = cblocks * cuda_block;
    LOGV(("size: %d, csize %d, cuda_block %d, cblocks %d",size,csize,cuda_block,cblocks))
    
    size_D = csize * csize * TENSORM * DIMS0 * sizeof( DOUBLE );
    size_coord = csize * DIMS1 * sizeof( DOUBLE );
    if ( hydro && !geyer_on_the_fly )
    {
        cudaMalloc( (void**)&d_D, size_D ); CCERR
    }
    else
    {
        cudaMalloc( (void**)&d_D, csize * DIMS0 * sizeof( DOUBLE ) ); CCERR
    }

    cudaMalloc( (void**)&d_coord, size_coord ); CCERR
    cudaMalloc( (void**)&d_scoord, size_coord ); CCERR

    h_X = (DOUBLE*) malloc( size_coord ); CHMEM(h_X)
    h_Rs0 = (DOUBLE*) malloc( size_coord ); CHMEM(h_Rs0)

    cudaMalloc( (void**)&d_X, size_coord ); CCERR
    cudaMalloc( (void**)&d_Rs0, size_coord ); CCERR
    cudaMalloc( (void**)&d_sRs0, size_coord ); CCERR

    if ( algorithm > DIFF_ALG_ERMAK_NEWTON )
    {
        cudaMalloc( (void**)&d_acoord, size_coord ); CCERR
        cudaMalloc( (void**)&d_aRs0, size_coord ); CCERR
        cudaMalloc( (void**)&d_aD, size_D ); CCERR
    }
    cudaMemcpy( d_coord, coord, size * DIMS1 * sizeof( DOUBLE ), cudaMemcpyHostToDevice ); CCERR
    if ( csize - size )
    {
        INT diff = csize - size;
        INT diffsize = diff*DIMS1*sizeof(DOUBLE);
        INT i;
        DOUBLE d = 0;
        DOUBLE* tc = (DOUBLE*) malloc( diffsize );
        d = 1/d;
        for ( i = 0; i < diff*DIMS1; ++i ) tc[i] = d;
        cudaMemcpy( d_coord + size*DIMS1, tc, diffsize, cudaMemcpyHostToDevice ); CCERR
        free( tc );
    }
}

void free_cuda()
{
    cudaFree( d_coord ); CCERR
    if ( hydro )
    {
        cudaFree( d_D ); CCERR
        if ( d_acoord ) { cudaFree( d_acoord ); CCERR }
        if ( d_aRs0 )   { cudaFree( d_aRs0   ); CCERR }
        if ( d_aD )     { cudaFree( d_aD     ); CCERR }
    }
    free( h_X );
    cudaFree( d_X ); CCERR
    free( h_Rs0 );
    cudaFree( d_Rs0 ); CCERR

    cublasShutdown(); CCERR
    free_cuda_common();
}

void init_cuda_after_restart()
{
    init_cuda();
}

#endif
