/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef CUDA_H
#define	CUDA_H

#include "data.h"

#include "../common/cuda_common.h"

#ifdef	__cplusplus
extern "C" {
#endif

#if USE_CUDA

#ifdef USE_FLOAT
#define DOUBLE_4 float4
#define DOUBLE_3 float3
#define DOUBLE_2 float2
#else
#define DOUBLE_4 double4
#define DOUBLE_3 double3
#define DOUBLE_2 double2
#endif

#define TENSORM DIMS0

/*! Number of blocks in CUDA kernel. */
extern INT cblocks;

/*! Aligned size. */
extern INT csize;

/*! Array of coordinates on device. */
extern DOUBLE* d_coord;

/*! Copy of coordinates on device. */
extern DOUBLE* d_scoord;

/*! Copy of forces on device. */
extern DOUBLE* d_sforces;

/*! Copy of forces on host. */
extern DOUBLE* h_sforces;

/*! Array of forces on device. */
extern DOUBLE* d_forces;

/*! Array of forces on host. */
extern DOUBLE* h_forces;

/*! Diffusion tensor matrix on device.
 *  May contain only diagonal elements in non-hi case. */
extern DOUBLE* d_D;

/*! Diffusion tensor matrix on host. */
extern DOUBLE* h_D;

/*! Vector of random numbers on device. */
extern DOUBLE* d_X;

/*! Vector of random numbers on host. */
extern DOUBLE* h_X;

/*! Vector of beads displacements on device. */
extern DOUBLE* d_Rs0;

/*! Vector of beads displacements on host. */
extern DOUBLE* h_Rs0;

/*! Copy of beads displacements on device. */
extern DOUBLE* d_sRs0;

/*! Sizeof diffusion tensor matrix. \sa d_D */
extern INT    size_D;

/*! Size of coordinates array. \sa d_coord */
extern INT    size_coord;

/*! Array of bonded forces values on device. */
extern DOUBLE* d_bforces;

/*! Duplicate of coordinates for IGT algorithm on device. */
extern DOUBLE* d_acoord;

/*! Duplicate of beads displacments for IGT algorithm on device. */
extern DOUBLE* d_aRs0;

/*! Duplicate of beads displacments for IGT algorithm on device. */
extern DOUBLE* d_aD;

/*! Initialize data for random number generation on device. */
void init_cuda_rand();

/*! Free resources on device. */
void free_cuda_rand();

/*! Initialize data for position computation on device. */
void init_cuda_equation();

/*! Free resources on device. */
void free_cuda_equation();
#endif

/*! Initializes CUDA device and allocs memory after restart. */
void init_cuda_after_restart();

/*! Initializes CUDA device and allocs memory. */
void init_cuda();

/*! Free memory and other resources used in CUDA. */
void free_cuda();

#ifdef	__cplusplus
}
#endif

#endif	/* CUDA_H */
