/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef DATA_H
#define DATA_H

#include "../common/defs.h"

#if USE_MPI
#include <mpi.h>
#endif

#ifdef	__cplusplus
extern "C" {
#endif

#if !HAVE_ERFC

/*! Error function, double precision version. Used mainly for windows platform. */
double erfc(double);

/*! Error function, single precision version. Used mainly for windows platform. */
float  erfcf(float);
#endif

typedef INT NO_CHOLS_GEYER;

#define SUB_TAG "sub"
#define BOND_TAG "bond"
#define ANGLE_TAG "angle"
#define DIHE_TAG "dihe"


/*! Types of energies in system. */
enum ENERGY_TYPE
{
    ENERGY_NONE,
    ENERGY_BOND,
    ENERGY_ANGLE,
    ENERGY_ANGLE_COS,
    ENERGY_DIHE,
    ENERGY_DIHE_ANGLE,
    ENERGY_LJ,
    ENERGY_COULOMB,
    ENERGY_LAST
};

/*! Number of beads */
extern INT size;
MAKE_STR_DEC_INNER(INT,size)

/*! Coordinates array - x,y,z,radii */
extern PTR_DOUBLE coord;
MAKE_STR_DEC_INNER(PTR_DOUBLE,coord)

/*! Optional masses of beads used in newton for collision. */
extern PTR_DOUBLE masses;
MAKE_STR_DEC_INNER(PTR_DOUBLE,masses)

/*! Charges of beads */
extern PTR_DOUBLE Q;
MAKE_STR_DEC_INNER(PTR_DOUBLE,Q)

/*! Lennard Jones variables */
extern PTR_DOUBLE LJ;
MAKE_STR_DEC_INNER(PTR_DOUBLE,LJ)

/*! Names of beads */
extern PTR_STR names;
MAKE_STR_DEC_INNER(PTR_STR,names)

/*! Ids of beads */
extern PTR_INT ids;
MAKE_STR_DEC_INNER(PTR_INT,ids)

/*! Copy of coordinates. */
extern DOUBLE* save_coord;

/*! Copy of forces. */
extern DOUBLE* save_forces;

/*! Convert id to array offset. */
INT resolve_pos(INT id);

/*! Add beads to coordinates, Q, LJ, names and ids arrays. */
void add_sub(CSTR namei, INT idi, DOUBLE xi, DOUBLE yi, DOUBLE zi, DOUBLE sigmai, DOUBLE Qi, DOUBLE sigma_LJi, DOUBLE epsilon_LJi, DOUBLE mass );

/*! Trim arrays to number of beads. */
void end_sub();

#ifdef	__cplusplus
}
#endif

#endif	/* DATA_H */
