/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@cent.uw.edu.pl>        *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef MATH_HELP_H
#define MATH_HELP_H

#include "data.h"

#ifdef __cplusplus
extern "C" {
#endif

/*! Computes distance between beads.
 *  \param p1 id of first bead
 *  \param p2 id of seconf bead
 *  \param r_sq square of distance */
void get_i2_sq( int p1, int p2, DOUBLE* r_sq );

/*! Computes distance between beads.
 *  \param p1 id of first bead
 *  \param p2 id of seconf bead
 *  \param e1 unit vector
 *  \param r_norm [out] distance between beads
 *  \param r_sq [out] square of distance */
void get_i2_norms( int p1, int p2, DOUBLE* e1, DOUBLE* r_norm, DOUBLE* r_sq );

/*! Computes normalized distance vectors between beads.
 *  \param p1 id of first bead
 *  \param p2 id of second bead
 *  \param p3 id of third bead
 *  \param e1 unit vector P2P1
 *  \param e2 unit vector P2P3 */
void get_i3_norms( int p1, int p2, int p3, DOUBLE* e1, DOUBLE* e2 );

/*! Normalize vector and computes it length.
 *  \param p1 id of first bead
 *  \param p2 id of seconf bead
 *  \param e1 [out] unit vector of (x1,y1,z1) vector
 *  \param r_norm [out] distance between beads
 *  \param r_sq [out] square of distance */
void get_i2_norms_no( int p1, int p2, DOUBLE* e1, DOUBLE* r_norm, DOUBLE* r_sq );

#ifdef	__cplusplus
}
#endif

#endif	/* MATH_HELP_H */
