/**************************************************************************
 *                                                                        *
 *   Copyright (C) 2010 - 2012: University of Warsaw                      *
 *   Authors:            Pawel Zielinski <pzielins@icm.edu.pl>            *
 *                       Maciej Dlugosz  <mdlugosz@icm.edu.pl>            *
 *                                                                        *
 *   This program is free software; you can redistribute it and/or modify *
 *   it under the terms of the GNU General Public License as published by *
 *   the Free Software Foundation; either version 3 of the License, or    *
 *   (at your option) any later version.                                  *
 *                                                                        *
 *   This program is distributed in the hope that it will be useful,      *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 *   GNU General Public License for more details.                         *
 *                                                                        *
 *   You should have received a copy of the GNU General Public License    *
 *   along with this program; if not, see http://www.gnu.org/licenses     *
 *   or write to the Free Software Foundation,Inc., 51 Franklin Street,   *
 *   Fifth Floor, Boston, MA 02110-1301  USA                              *
 *                                                                        *
 **************************************************************************/

#ifndef BUCKETF_H
#define BUCKETF_H

#include "../data.h"

#ifdef __cplusplus
extern "C" {
#endif

#define BUCKET_N 16
#define BUCKET_CAP_MLT 1
#define BUCKET_CAP_ADD 8

#define BUCKET_DIM 3

/*! Algorith to generate interaction list in spatial subdivision. */
extern BUCKET_NONE_SPATIAL nb_list;
MAKE_STR_DEC(BUCKET_NONE_SPATIAL,nb_list)
extern DOUBLE max_force_cutoff;

typedef struct
{
    int x,y,z,size,capacity,*ids;
} reg_bucket;

typedef struct
{
    int size;
    reg_bucket* regs;
} list_reg;

typedef struct
{
    DOUBLE blen  [DIMS0];
    DOUBLE offset[DIMS0];
    INT mlocal[DIMS0];
    list_reg lreg[BUCKET_N][BUCKET_N][BUCKET_N];
} bucket_f;

bucket_f* init_bucket_f( int size, DOUBLE4* coords, DOUBLE* radii, DOUBLE blen );
void get_bid_f( bucket_f* br, DOUBLE4* bead, int* p, int* local );
void add_coord_f( bucket_f* br, int id, DOUBLE4* bead );
void free_bucket_f( bucket_f* br, int size, DOUBLE4* coord );
void print_statistic_f( bucket_f* br );
void get_neighbours( bucket_f* br, DOUBLE4* bead, int sizes[64], int* ids[64] );

#ifdef	__cplusplus
}
#endif

#endif/* BUCKET_H */
